<?php

namespace Drupal\flag_lists\Controller;

use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\EventSubscriber\MainContentViewSubscriber;
use Drupal\Core\Render\RendererInterface;
use Drupal\Core\Url;
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\Component\Utility\UrlHelper;
use Drupal\flag\Ajax\ActionLinkFlashCommand;
use Drupal\flag\Controller\ActionLinkController as FlagActionLinkController;
use Drupal\flag\FlagInterface;
use Drupal\flag\FlagServiceInterface;
use Drupal\flag_lists\Entity\FlagListItem;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Request;
use Drupal\Component\Utility\Html;

/**
 * Controller responses to flag and unflag action links.
 *
 * If the action_link is a normal link then after an update the response to a
 *  valid request is a redirect to the entity with drupal update message.
 *
 * For an ajax_action_link the response is a set of AJAX commands to update the
 * link in the page. If the user agent has javascript disabled then the
 * behaviour reverts to that of a normal link.
 *
 */

class ActionLinkController extends FlagActionLinkController {
  /**
   * The flag service.
   *
   * @var \Drupal\flag\FlagServiceInterface
   */
  protected $flagService;

  /**
   * The renderer service.
   *
   * @var \Drupal\Core\Render\RendererInterface
   */
  protected $renderer;

  /**
   * Constructor.
   *
   * @param \Drupal\flag\FlagServiceInterface $flag
   *   The flag service.
   * @param \Drupal\Core\Render\RendererInterface $renderer
   *   The Renderer service.
   */
  public function __construct(FlagServiceInterface $flag, RendererInterface $renderer) {
    $this->flagService = $flag;
    $this->renderer = $renderer;
  }

  /**
   * {@inheritdoc}
   */
public static function create(ContainerInterface $container) {
    return new static(
      $container->get('flag'),
      $container->get('renderer')
    );
  }

  /**
   * @see \Drupal\flag_lists\Entity\FlagListItem
   *
   * @param \Drupal\flag\FlagInterface $flag
   *   The flag entity.
   * @param int $entity_id
   *   The flaggable entity ID.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse|\Symfony\Component\HttpFoundation\RedirectResponse|null
   *   The response object, only if successful.
   *
   * @see \Drupal\flag\Plugin\Reload
   * Performs a flagging when called via a route.
   * In addition to this it also handles the creation of a
   * FlagListItem.
   */
  public function flag(FlagInterface $flag, $entity_id, Request $request) {
    /* @var \Drupal\Core\Entity\EntityInterface $entity */
    $entity = $this->flagService->getFlaggableById($flag, $entity_id);

    try {
      $this->flagService->flag($flag, $entity);
    }
    catch (\LogicException $e) {
      // Fail silently so we return to the entity, which will show an updated
      // link for the existing state of the flag.
    }
    $flag_list_service = \Drupal::service('flaglists');
    $flag_list = $request->attributes->get('flag_list');
    $flag_list_name =
      $flag_list_service->getFlaggingCollectionById($flag_list)->getName();
    $flag_list_item = FlagListItem::create([
      'entity_id' => $entity_id,
      'type' => $flag->getFlaggableEntityTypeId(),
      'baseflag' => $flag->id(),
      'flag_list' => $flag_list,
      'name' => $flag_list_name . ' ' . $entity_id,
    ]);
    $flag_list_item->save();

    return $this->generateResponse($flag, $entity, $request, $flag->getMessage('flag'));
  }

  /**
   * @see \Drupal\flag_lists\Entity\FlagListItem
   *
   * @param \Drupal\flag\FlagInterface $flag
   *   The flag entity.
   * @param int $entity_id
   *   The flaggable entity ID.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse|\Symfony\Component\HttpFoundation\RedirectResponse|null
   *   The response object, only if successful.
   *
   * @see \Drupal\flag\Plugin\Reload
   * Performs a unflagging when called via a route.
   * In addition to this it also handles the deletion of the
   * related FlagListItem's.
   */
  public function unflag(FlagInterface $flag, $entity_id, Request $request) {
    $flag_lists_service = \Drupal::service('flaglists');
    /* @var \Drupal\Core\Entity\EntityInterface $entity */
    $entity = $this->flagService->getFlaggableById($flag, $entity_id);

    try {
      $this->flagService->unflag($flag, $entity);
    }
    catch (\LogicException $e) {
      // Fail silently so we return to the entity, which will show an updated
      // link for the existing state of the flag.
    }
    $baseflag = $flag->id();
    $flag_list = $request->attributes->get('flag_list');
    $ids = $flag_lists_service->getFlagListItemIds($baseflag, $flag_list,
      $entity_id);
    foreach ($ids as $id) {
      $flagListItem = \Drupal::entityTypeManager()->getStorage('flag_list_item')->load($id);
      $flagListItem->delete();
    }

    return $this->generateResponse($flag, $entity, $request, $flag->getMessage('unflag'));
  }

  /**
   * @see \Drupal\flag_lists\Entity\FlaggingCollection
   *
   * @param \Drupal\flag\FlagInterface $flag
   *   The flag entity.
   * @param \Drupal\Core\Entity\EntityInterface $entity
   *   The entity object.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object. 
   * @param string $message
   *   (optional) The message to flash.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse|\Symfony\Component\HttpFoundation\RedirectResponse
   *   The response object.
   *
   * Generates a response after the flag has been updated.
   * This response is different from the Flag modules
   * response as we add the flagging collection Id as well.
   */
  protected function generateResponse(FlagInterface $flag, EntityInterface $entity, Request $request, $message = NULL) {
    if ($request->get(MainContentViewSubscriber::WRAPPER_FORMAT) == 'drupal_ajax') {
      $flag_list = $request->attributes->get('flag_list');
      // Create a new AJAX response.
      $response = new AjaxResponse();

      // Get the link type plugin.
      $link_type = $flag->getLinkTypePlugin();

      // Generate the link render array.
      $link = $link_type->getAsFlagLink($flag, $entity);
      $flagListService = \Drupal::service('flaglists');
      $flc = $flagListService->getFlaggingCollectionById($flag_list);
      $token_service = \Drupal::token();
      $bubbleable_metadata = new BubbleableMetadata();
      $link['#title']['#markup'] =
        $token_service->replace($link['#title']['#markup'],
          ['flagging_collection' => $flc], [], $bubbleable_metadata);

      $bubbleable_metadata
        ->applyTo($link);

      $link['#flagging_collection'] = $flag_list;

      // Add the flag_list to the link
      // This is a messy method where everything is made by hand...
      $options = UrlHelper::parse($link['#attributes']['href']);
      $uri = 'internal:' . $options['path'] . '/' . $flag_list;
      $path = ltrim($options['path'] . '/' . $flag_list, '/');
      // Create a token.
      $generator = \Drupal::csrfToken();
      $token = $generator->get($path);
      unset($options['path']);
      $options['query']['token'] = $token;
      // Update the href.
      $link['#attributes']['href'] = Url::fromUri($uri, $options)->toString();

      // Generate a CSS selector to use in a JQuery Replace command.
      $selector = '.js-flag-' . Html::cleanCssIdentifier($flag->id()) . '-' . $entity->id() . '-' . $flag_list;
      // Create a new JQuery Replace command to update the link display.
      $replace = new ReplaceCommand($selector, $this->renderer->renderPlain($link));
      $response->addCommand($replace);

      if ($message) {
        // Push a message pulsing command onto the stack.
        $pulse = new ActionLinkFlashCommand($selector, $message);
        $response->addCommand($pulse);
      }
    }
    elseif ($entity->hasLinkTemplate('canonical')) {
      // Redirect back to the entity. A passed in destination query parameter
      // will automatically override this.
      $url_info = $entity->toUrl();

      $options['absolute'] = TRUE;
      $url = Url::fromRoute($url_info->getRouteName(), $url_info->getRouteParameters(), $options);
      $response = new RedirectResponse($url->toString());

    }
    else {
      // For entities that don't have a canonical URL (like paragraphs),
      // redirect to the front page.
      $front = Url::fromUri('internal:/');
      $response = new RedirectResponse($front);
    }

    return $response;
  }

}
