<?php

namespace Drupal\flag_lists;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\Query\QueryFactory;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Session\SessionManagerInterface;
use Drupal\user\UserInterface;

/**
 * Class FlagForList service.
 *  - Handles search requests for flags and flaggings.
 *  - Performs flagging and unflaging operations.
 *  - Most of it SWP (Stolen With Pride) from flag module
 */
class FlagListsService implements FlagListsServiceInterface {

  /**
   * The entity query manager injected into the service.
   *
   * @var \Drupal\Core\Entity\Query\QueryFactory
   */
  private $entityQueryManager;

  /**
   * The current user injected into the service.
   *
   * @var AccountInterface
   */
  private $currentUser;

  /*
   * The type Entity Type Manager injected into the service
   *
   * @var EntityTypeManagerInterface
   * */
  private $entityTypeManager;

  /*
   * The session manager.
   *
   * @var Drupal\Core\Session\SessionManagerInterface
   */
  private $sessionManager;

  /**
   * Constructor.
   *
   * @param QueryFactory $entity_query
   *   The entity query factory.
   * @param AccountInterface $current_user
   *   The current user.
   * @param EntityTypeManagerInterface $entity_type_manager
   *   The entity manager.
   * @param Drupal\Core\Session\SessionManagerInterface $session_manager
   *   The session manager.
   */
  public function __construct(QueryFactory $entity_query,
                              AccountInterface $current_user,
                              EntityTypeManagerInterface $entity_type_manager,
                              SessionManagerInterface $session_manager) {
    $this->entityQueryManager = $entity_query;
    $this->currentUser = $current_user;
    $this->entityTypeManager = $entity_type_manager;
    $this->sessionManager = $session_manager;
  }

  /**
   * Get all Flag For List, i.e. all Flags tagged as available
   * as templates.
   *
   * @param $entity_type
   *   The entity wanted entity type.
   * @param $bundle
   *   The wanted bundle of the entity.
   *
   * @return Drupal\flag_lists\Entity\FlagForList array
   *   An array of found Flag For Lists
   */
  public function getAllFlagForList($entity_type = NULL, $bundle = NULL) {
    $query = $this->entityQueryManager->get('flag_for_list');

    if ($entity_type != NULL) {
      $query->condition('entity_type', $entity_type);
    }

    $ids = $query->execute();
    $flags = $this->getFlagForListByIds($ids);

    if (isset($bundle)) {
      $flags = array_filter($flags, function (FlagForList $flag) use ($bundle) {
        $bundles = $flag->getApplicableBundles();
        return in_array($bundle, $bundles);
      });
    }

    return $flags;
  }

  /**
   * Get current user's Flagging Collections
   *
   * @return Drupal\flag_lists\Entity\FlaggingCollection array
   *   An array of current User's Flagging Collections
   */
  public function getUsersFlaggingCollections() {
    $query = $this->entityQueryManager->get('flagging_collection');
    $query->condition('user_id', $this->currentUser->id());

    $ids = $query->execute();
    $flags = $this->getFlaggingCollectionByIds($ids);

    return $flags;
  }

  /**
   * Get all Flagging Collections
   *
   * @param type
   *   The type, bundle, of the Flagging Collection
   *
   * @return Drupal\flag_lists\Entity\FlaggingCollection array
   *   An array of the found Flagging Collections
   */
  public function getAllFlaggingCollections($type = NULL) {
    $query = $this->entityQueryManager->get('flagging_collection');

    if ($type != NULL) {
      $query->condition('type', $type);
    }

    $ids = $query->execute();
    $flags = $this->getFlaggingCollectionByIds($ids);

    return $flags;
  }

  /**
   * Get a user's Flag For List, i.e. all Flags tagged as available
   * as templates.
   *
   * NOT used nor tested!
   *
   * @param AccountInterface $account
   *   The account of the wanted user.
   * @param $entity_type
   *   The entity wanted entity type.
   * @param $bundle
   *   The wanted bundle of the entity.
   *
   * @return Drupal\flag_lists\Entity\FlagForList array
   *   An array of Flag For Lists that the account has access to
   */
  public function getUsersFlagForList(AccountInterface $account, $entity_type = NULL, $bundle = NULL) {
    $flags = $this->getAllFlagForList($entity_type, $bundle);

    $filtered_flags = [];
    foreach ($flags as $flag_id => $flag) {
      if ($flag->actionAccess('flag', $account)->isAllowed() ||
          $flag->actionAccess('unflag', $account)->isAllowed()) {
        $filtered_flags[$flag_id] = $flag;
      }
    }

    return $filtered_flags;
  }

  /**
   * Get a specific Flagging Collections
   *
   * NOT used nor tested!
   *
   * @param FlagForListInterface $flag
   *   The flag list?
   * @param EntityInterface $entity
   *   The entity?
   * @param AccountInterface $account
   *   The account of the wanted user.
   * @param $isession_id
   *   The session id
   *
   * @return Drupal\flag_lists\Entity\FlaggingCollection array
   *   An array of the found Flagging Collections
   */
  public function getFlaggingCollection(FlagForListInterface $flag, EntityInterface $entity, AccountInterface $account = NULL, $session_id = NULL) {
    $this->populateFlaggerDefaults($account, $session_id);

    $flaggings = $this->getEntityFlaggings($flag, $entity, $account, $session_id);

    return !empty($flaggings) ? reset($flaggings) : NULL;
  }

  /**
   * NOT used nor tested!
   *
   * {@inheritdoc}
   */
  public function populateFlaggerDefaults(AccountInterface &$account = NULL, &$session_id = NULL) {
    // Note that the $account parameter must be explicitly set to be passed by
    // reference for the case when the variable is NULL rather than an object;
    // also, it must be optional to allow a variable that is NULL to pass the
    // type-hint check.

    // Get the current user if the account is NULL.
    if ($account == NULL) {
      $account = $this->currentUser;

      // If the user is anonymous, get the session ID.
      if ($account->isAnonymous()) {
        // Ensure something is in $_SESSION, otherwise the session ID will
        // not persist.
        // TODO: Replace this with something cleaner once core provides it.
        // See https://www.drupal.org/node/2865991.
        $_SESSION['flag_lists'] = TRUE;

        $this->sessionManager->start();

        // Intentionally clobber $session_id; it makes no sense to specify that
        // but not $account.
        $session_id = $this->sessionManager->getId();
      }
    }
    elseif ($account->isAnonymous() && is_null($session_id)) {
      throw new \LogicException('Anonymous users must be identifed by session_id');
    }
  }

  /**
   * Get a Flag For List, i.e. the Flags tagged as available
   * as templates.
   *
   * @param $flag_id
   *   The template id to load
   *
   * @return Drupal\flag_lists\Entity\FlagForList array
   *   An array of Flag For Lists
   */
  public function getFlagForListById($flag_id) {
    return $this->entityTypeManager->getStorage('flag_for_list')->load($flag_id);
  }

  /**
   * Get a Flagging Collection by Id
   *
   * @param flag_id
   *   The flag Id of the Flagging Collection to load
   *
   * @return Drupal\flag_lists\Entity\FlaggingCollection
   *   The loaded Flagging Collection
   */
  public function getFlaggingCollectionById($flag_id) {
    return $this->entityTypeManager->getStorage('flagging_collection')->load($flag_id);
  }

  /**
   * Loads Flag For Lists by ids
   *
   * @param int[] $ids
   *   The Ids of the Flag For Lists to load
   *
   * @return Drupal\flag_lists\Entity\FlagForList array
   *   The loaded Flag For Lists
   */
  protected function getFlagForListByIds(array $ids) {
    return $this->entityTypeManager->getStorage('flag_for_list')->loadMultiple($ids);
  }

  /**
   * Loads Flagging Collections by ids
   *
   * @param int[] $ids
   *   The Ids of the Flag For Lists to load
   *
   * @return Drupal\flag_lists\Entity\FlaggingCollection
   *   The loaded Flagging Collections
   */
  protected function getFlaggingCollectionByIds(array $ids) {
    return $this->entityTypeManager->getStorage('flagging_collection')->loadMultiple($ids);
  }

  /**
   * Load a Flagging Collection Type by Id
   *
   * @param type_id
   *   The Id of the Flagging Collection Type to load
   *
   * @return Drupal\flag_lists\Entity\FlaggingCollectionType
   *   The loaded Flagging Collection Type
   */
  public function getFlaggingCollectionTypeById($type_id) {
    return $this->entityTypeManager->getStorage('flagging_collection_type')->load($type_id);
  }

  /**
   * Get Flag For List ids
   *
   * @param $base_flag
   *   The base flag to use
   * @param $flag_list
   *   The flag list
   * @param $entity_id
   *   The entity id
   *
   * @return int[] $ids
   *   An array of ids for the Flag For Lists
   */
  public function getFlagListItemIds($baseflag, $flag_list = NULL,
    $entity_id = NULL ) {
    $query = $this->entityQueryManager->get('flag_list_item');
    $query->condition('baseflag', $baseflag);
    if ( $flag_list <> NULL ) {
      $query->condition('flag_list', $flag_list);
    }
    if ( $entity_id <> NULL ) {
      $query->condition('entity_id', $entity_id);
    }
    $ids = $query->execute();
    return $ids;
  }

  /**
   * Get Flag List Items
   *
   * @param int[] $ids
   *   The Flag List Item ids to load
   *
   * @return Drupal\flag_lists\Entity\FlagListItem array
   *   An array of Flag For Lists
   */
  public function getFlagListItems($ids) {
    return $this->entityTypeManager->getStorage('flag_list_item')->loadMultiple($ids);
  }

  /**
   * Get Flaggging Collection ids by name
   *
   * NOT used nor tested!
   *
   * @param relatedName
   *   The name of the Flagging Collection to load
   *
   * @return int[] id
   *   The Flagging Collection id
   */
  public function getFlaggingCollectionIdByName($relatedName) {
    $query = $this->entityQueryManager->get('flagging_collection');
    // The name is actually the same as the relatedName
    $query->condition('relatedflag', $relatedName);
    $id = $query->execute();
    return $id;
  }
}
